
// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2022 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

/************************************************************************
 *     DATE:        16/03/2006                                          *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT:  Définir des classes d'exception pour la gestion d'erreur   *
 *           concernant les lois de comportements.                      *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *                                                                      *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/
#ifndef EXCEPTIONSLOICOMP_H
#define EXCEPTIONSLOICOMP_H


/// @addtogroup Groupe_concernant_le_chargement
///  @{
///

/// cas d'une erreur survenue à cause d'une non convergence pour la résolution
/// d'une loi de comportement incrémentale 

class ErrNonConvergence_loiDeComportement 
   // =0 cas courant, pas d'information particulière
   // =1 cas où l'erreur est sévère et ne pourra pas être corrigé en refaisant un calcul avec un
   //    pas de temps plus petit. Il faut refaire le calcul en se positionnant plusieurs pas de temps
   //    auparavant (utilisé par l'hystérésis par exemple) 
   { public :
      int cas;   
      ErrNonConvergence_loiDeComportement () : cas(0) {} ;  // par défaut 
      ErrNonConvergence_loiDeComportement (int ca) : cas(ca) {} ;  // pb 
    };
 /// @}  // end of group


#endif  
