

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2022 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

/************************************************************************
 *     DATE:        1/10/98                                             *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT: Concernant le potentiel hyperélastique proposé par denis    *
 *          favier, pour modéliser le comportement superélastique et    *
 *          ferroélastique des AMF.                                     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *                                                                      *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/
#ifndef ISOHYPER3DFAVIER3_H
#define ISOHYPER3DFAVIER3_H

#include "Hyper3D.h"


/// @addtogroup Les_lois_hyperelastiques
///  @{
///


class IsoHyper3DFavier3 :  public  Hyper3D
{
  public :
    // VARIABLES PUBLIQUES :
    
 // CONSTRUCTEURS :
 // Constructeur par defaut
 IsoHyper3DFavier3 ();
 // Constructeur de copie
 IsoHyper3DFavier3 (const IsoHyper3DFavier3& loi) ;
 // DESTRUCTEUR :
 ~IsoHyper3DFavier3 () 
   {};
  
 // initialise les donnees particulieres a l'elements
 // de matiere traite ( c-a-dire au pt calcule)
 // Il y a creation d'une instance de SaveResul particuliere
 // a la loi concernee
 // la SaveResul classe est remplie par les instances heritantes
 // le pointeur de SaveResul est sauvegarde au niveau de l'element
 // c'a-d que les info particulieres au point considere sont stocke
 // au niveau de l'element et non de la loi.
 class SaveResulIsoHyper3DFavier3: public SaveResulHyperD
  { public :
        SaveResulIsoHyper3DFavier3(bool avec_para = false); // constructeur par défaut :
        SaveResulIsoHyper3DFavier3(const SaveResulIsoHyper3DFavier3& sav); // de copie
        virtual ~SaveResulIsoHyper3DFavier3(); // destructeur
        // définition d'une nouvelle instance identique
        // appelle du constructeur via new
        virtual SaveResul * Nevez_SaveResul() const{return (new SaveResulIsoHyper3DFavier3(*this));};
        // affectation
        virtual SaveResul & operator = ( const SaveResul & a);
        //============= lecture écriture dans base info ==========
           // cas donne le niveau de la récupération
           // = 1 : on récupère tout
           // = 2 : on récupère uniquement les données variables (supposées comme telles)
        virtual void Lecture_base_info (istream& ent,const int cas);
           // cas donne le niveau de sauvegarde
           // = 1 : on sauvegarde tout
           // = 2 : on sauvegarde uniquement les données variables (supposées comme telles)
        virtual void Ecriture_base_info(ostream& sort,const int cas);
   
        // mise à jour des informations transitoires
        virtual void TdtversT();
        virtual void TversTdt();
  
        // affichage à l'écran des infos
        virtual void Affiche() const;
  
        //changement de base de toutes les grandeurs internes tensorielles stockées
        // beta(i,j) represente les coordonnees de la nouvelle base naturelle gpB dans l'ancienne gB
        // gpB(i) = beta(i,j) * gB(j), i indice de ligne, j indice de colonne
        // gpH(i) = gamma(i,j) * gH(j)
        virtual void ChBase_des_grandeurs(const Mat_pleine& beta,const Mat_pleine& gamma)
            {SaveResulHyperD::ChBase_des_grandeurs(beta,gamma);};
  
        // procedure permettant de completer éventuellement les données particulières
        // de la loi stockées
        // au niveau du point d'intégration par exemple: exemple: un repère d'anisotropie
        // completer est appelé apres sa creation avec les donnees du bloc transmis
        // peut etre appeler plusieurs fois
        virtual SaveResul* Complete_SaveResul(const BlocGen & bloc, const Tableau <Coordonnee>& tab_coor
                                      ,const Loi_comp_abstraite* loi)
            {return SaveResulHyperD::Complete_SaveResul(bloc,tab_coor,loi);};

  
        //-------------------------------------------------------------------
        // données: les paramètres matériaux réellement utilisés
        //-------------------------------------------------------------------
        // les 4 paramètres de la loi dans l'ordre suivant
        // double K,Qor,mur,mu_inf; // paramètres de la loi
        // ces paramètres ne sont définis "que si" on a une ou plusieurs fct nD
        // de manière à optimiser la place mémoire
        Vecteur *para_loi, *para_loi_t;
//        double K,K_t;           // module
//        double Qor,Qor_t;       // seuil
//        double mur,mur_t;       // pente à l'origine
//        double mu_inf,mu_inf_t; // pente à l'infini
  
  
         // ---- fin gestion d'une liste de grandeurs quelconques éventuelles ---
  };

   virtual SaveResul * New_et_Initialise()
     { bool avec_para = false;
       if (  (K_nD != NULL) || (Qor_nD != NULL)
          || (mur_nD != NULL) || (mu_inf_nD != NULL)
          )
          avec_para = true;
       SaveResulIsoHyper3DFavier3 * pt = new SaveResulIsoHyper3DFavier3(avec_para);
       // insertion éventuelle de conteneurs de grandeurs quelconque
       Insertion_conteneur_dans_save_result(pt);
       return pt;
     };
  
   friend class SaveResulIsoHyper3DFavier3;

 // Lecture des donnees de la classe sur fichier
 void LectureDonneesParticulieres (UtilLecture * ,LesCourbes1D& lesCourbes1D
              ,LesFonctions_nD& lesFonctionsnD);

 // affichage de la loi
 void Affiche() const ;
 // test si la loi est complete
 // = 1 tout est ok, =0 loi incomplete
 int TestComplet();

	 
//----- lecture écriture de restart -----
 // cas donne le niveau de la récupération
 // = 1 : on récupère tout
 // = 2 : on récupère uniquement les données variables (supposées comme telles)
 void Lecture_base_info_loi(istream& ent,const int cas,LesReferences& lesRef,LesCourbes1D& lesCourbes1D
                                             ,LesFonctions_nD& lesFonctionsnD);
 // cas donne le niveau de sauvegarde
 // = 1 : on sauvegarde tout
 // = 2 : on sauvegarde uniquement les données variables (supposées comme telles)
 void Ecriture_base_info_loi(ostream& sort,const int cas);

 // récupération des grandeurs particulière (hors ddl )
 // correspondant à liTQ
 // absolue: indique si oui ou non on sort les tenseurs dans la base absolue ou une base particulière
 virtual void Grandeur_particuliere
      (bool absolue,List_io<TypeQuelconque>& ,Loi_comp_abstraite::SaveResul * ,list<int>& decal) const ;
 // récupération de la liste de tous les grandeurs particulières
 // ces grandeurs sont ajoutées à la liste passées en paramètres
 // absolue: indique si oui ou non on sort les tenseurs dans la base absolue ou une base particulière
 virtual void ListeGrandeurs_particulieres(bool absolue,List_io<TypeQuelconque>& ) const;

 // calcul d'un module d'young équivalent à la loi, ceci pour un
 // chargement nul
 double Module_young_equivalent(Enum_dure temps,const Deformation & def,SaveResul * saveDon);

 // récupération d'un module de compressibilité équivalent à la loi,
 // il s'agit ici de la relation -pression = sigma_trace/3. = module de compressibilité * I_eps
 double Module_compressibilite_equivalent(Enum_dure temps,const Deformation & def,SaveResul * saveDon);
 
  // récupération de la variation relative d'épaisseur calculée: h/h0
  //  cette variation n'est utile que pour des lois en contraintes planes
  // - pour les lois 3D : retour d'un nombre très grand, indiquant que cette fonction est invalide
  // - pour les lois 2D def planes: retour de 0
  // les infos nécessaires à la récupération , sont stockées dans saveResul
  // qui est le conteneur spécifique au point où a été calculé la loi
  virtual double HsurH0(SaveResul * saveResul) const {return ConstMath::tresgrand;};
 
  // création d'une loi à l'identique et ramène un pointeur sur la loi créée
  Loi_comp_abstraite* Nouvelle_loi_identique() const  { return (new IsoHyper3DFavier3(*this)); };
              
  // affichage et definition interactive des commandes particulières à chaques lois
  void Info_commande_LoisDeComp(UtilLecture& lec);
 
//  METHODES internes spécifiques à l'hyperélasticité isotrope découlant de 
//  méthodes virtuelles de Hyper3D


  // calcul du potentiel tout seul sans la phase car Qeps est  nul
  // ou très proche de 0
  double PoGrenoble
             (const double & Qeps,const Invariant & invariant);
  // calcul du potentiel tout seul avec la phase donc dans le cas où Qeps est non nul
  double PoGrenoble
           (const Invariant0QepsCosphi & inv,const Invariant & invariant);
  // calcul du potentiel tout seul sans la phase car Qeps est  nul
  // ou très proche de 0, et de sa variation suivant V uniquement
  PoGrenoble_V PoGrenoble_et_V
           (const double & Qeps,const Invariant & invariant);
  // calcul du potentiel et de sa variation suivant V uniquement
  PoGrenoble_V PoGrenoble_et_V
           (const Invariant0QepsCosphi & inv,const Invariant & invariant);
  // calcul du potentiel tout seul sans la phase car Qeps est  nul
  // ou très proche de 0, et de ses variations première et seconde suivant V uniquement
  PoGrenoble_VV PoGrenoble_et_VV
           (const double & Qeps,const Invariant & invariant);
  // calcul du potentiel et de sa variation première et seconde suivant V uniquement
  PoGrenoble_VV PoGrenoble_et_VV
           (const Invariant0QepsCosphi & inv,const Invariant & invariant);
  // calcul du potentiel et de ses dérivées non compris la phase
  PoGrenobleSansPhaseSansVar PoGrenoble
           (const InvariantQeps & inv,const Invariant & invariant);
  // calcul du potentiel et de ses dérivées avec la phase
  PoGrenobleAvecPhaseSansVar PoGrenoblePhase
           (const InvariantQepsCosphi& inv,const Invariant & invariant); 
  // calcul  du potentiel sans phase et dérivées avec  ses variations par rapport aux invariants
  PoGrenobleSansPhaseAvecVar PoGrenoble_et_var
           (const Invariant2Qeps& inv,const Invariant & invariant);
  // calcul  du potentiel avec phase et dérivées avec  ses variations par rapport aux invariants
  PoGrenobleAvecPhaseAvecVar PoGrenoblePhase_et_var
           (const Invariant2QepsCosphi& inv,const Invariant & invariant);

 
protected :

  // VARIABLES PROTEGEES :
  double  K;     // module de dilatation
  double  Qor;    // seuil
  double  mur;   // pente à l'origine
  double  mu_inf;   // pente à l'infini 
  double nQor,gammaQor; // paramètres de phase pour Qor
  double n_mu_inf,gamma_mu_inf; // paramètre de phase pour mu_inf
  // cas d'une dépendance à une fonction nD
  // -- bien voir que dans ce cas on peut obtenir n'importe quoi, c'est de la responsabilité
  //    de l'utilisateur de savoir se qu'il fait
  Fonction_nD * K_nD, * Qor_nD, * mur_nD, * mu_inf_nD;

  static double limite_co2; // limite à partir de laquelle on considère que cosh(co2) = infini
    
  ///=============== fonctions pour la vérification et la mise au point ===============
      
      // vérif des dérivées du potentiels par rapport aux invariants, ceci par différences finies
  void Verif_PoGrenoble_et_var(const double & Qeps,const Invariant & inv
                       ,const PoGrenobleSansPhaseAvecVar& potret );
      // idem mais avec la phase
  void Verif_PoGrenoble_et_var(const double & Qeps,const Invariant & inv,const double& cos3phi 
                       ,const PoGrenobleAvecPhaseAvecVar& potret );

  static int indic_Verif_PoGrenoble_et_var; // indicateur utilisé par Verif_Potentiel_et_var                           

 };
 /// @}  // end of group

#endif  
